#define USE_ARDUINO_INTERRUPTS true
#include <Stepper.h>
#include <LiquidCrystal_I2C.h>
#include <SHT21.h>
#include <Servo.h>
#include <PulseSensorPlayground.h>

const int OUTPUT_TYPE = SERIAL_PLOTTER;
const int PULSE_INPUT = A0;
const int PULSE_BLINK = 5;
const int THRESHOLD = 300;

PulseSensorPlayground pulseSensor;

Servo heart;
const int SERVO_PIN = 6;
int pos = 45;

LiquidCrystal_I2C lcd(0x27, 16, 2);

const int stepsPerRevolution = 2038;
Stepper myStepper = Stepper(stepsPerRevolution, 12, 10,11, 9);

SHT21 sht;
float Humidity;

bool start = false;
int i = 0;
int v[20];
float averageHeartRate;
int myBPM;

const int buttonPin = 4;
int buttonState = 0;

void setup() {
  Serial.begin(9600);
  Wire.begin();

  lcd.init();
  lcd.backlight();

  lcd.setCursor(0,0);
  lcd.print("Ready?");

  heart.attach(SERVO_PIN);
  heart.write(pos);

  pulseSensor.analogInput(PULSE_INPUT);
  pulseSensor.blinkOnPulse(PULSE_BLINK);

  pulseSensor.setOutputType(OUTPUT_TYPE);
  pulseSensor.setThreshold(THRESHOLD);

  pinMode(buttonPin, INPUT);

  pinMode(7, OUTPUT);
  pinMode(8, OUTPUT);

  if (!pulseSensor.begin()) {
    lcd.setCursor(0,0);
    lcd.print("Pulse error");
    for(;;) {
      // Flash the led to show things didn't work.
      digitalWrite(PULSE_BLINK, LOW);
      delay(50);
      digitalWrite(PULSE_BLINK, HIGH);
      delay(50);
    }
  }
}

void loop() {
  buttonState = digitalRead(buttonPin);

  if(buttonState == HIGH) {
    start = true;
  }

  delay(20);
  pulseSensor.outputSample();
  myBPM = pulseSensor.getLatestSample();
  moveServo(myBPM);

  if(start == true) {
    // myStepper.setSpeed(15);
	  // myStepper.step(stepsPerRevolution);
    if(i < 20) {
      if (myBPM > 60 && myBPM < 150) {
        Serial.print(i);
        Serial.print(". BPM: ");
        Serial.println(myBPM);
        v[i] = myBPM;
        i++;
      }
    } else if(i == 20) {
      int sum = 0;
      for (int j = 0; j < 20; j++)
        sum += v[j];
      averageHeartRate = sum / 20;
      Serial.print("Average: ");
      Serial.println(averageHeartRate);
      i++;
    } else {
      Humidity = sht.getHumidity();
      if(myBPM > 60 && myBPM < 150) {
        Serial.print("BPM: ");
        Serial.println(myBPM);
        if (Humidity > 60 || (averageHeartRate + 15 < myBPM 
                              || averageHeartRate - 15 > myBPM)) {
          lcd.clear();
          lcd.setCursor(0,0);
          lcd.print("Liar"); 
          digitalWrite(7, LOW);
          digitalWrite(8, HIGH);               
        } else {
          lcd.clear();
          lcd.setCursor(0,0);
          lcd.print("Telling the"); 
          lcd.setCursor(0,1);
          lcd.print("truth?");
          digitalWrite(8, LOW);
          digitalWrite(7, HIGH);
        }
        delay(3000);
      }
      i++;
    }
  }
}

void moveServo(int value){
  pos = map(value,0,1023,45,135);
  heart.write(pos);
}